#!/usr/bin/python

##############################################################################
#
# MODULE:       r.slope.stability.py
#
# AUTHOR:       Martin Mergili
# CONTRIBUTORS: Massimiliano Alvioli, Ivan Marchesini
#
# PURPOSE:      The slope stability model
#
# COPYRIGHT:    (c) 2013 - 2017 by the author
#               (c) 2013 - 2017 by the BOKU University, Vienna
#               (c) 2015 - 2017 by the University of Vienna
#               (c) 2013 - 2017 by the CNR-IRPI Perugia
#               (c) 2000 - 2017 by the GRASS Development Team
#
# VERSION:      20170406 (6 April 2017)
#
#               This program is free software under the GNU General Public
#               License (>=v2). Read the file COPYING that comes with GRASS
#               for details.
#
##############################################################################

#%module
#% description: The slope stability model
#% keywords: raster
#% keywords: slope stability
#%end

#%flag
#% key: a
#% description: Additional output rasters
#% guisection: flags
#%end

#%flag
#% key: c
#% description: Soil class mode
#% guisection: flags
#%end

#%flag
#% key: d
#% description: Documentation
#% guisection: flags
#%end

#%flag
#% key: i
#% description: Infinite slope stability mode
#% guisection: flags
#%end

#%flag
#% key: l
#% description: Layer mode
#% guisection: flags
#%end

#%flag
#% key: m
#% description: Multi-core processing
#% guisection: flags
#%end

#%flag
#% key: p
#% description: Slope failure probability
#% guisection: flags
#%end

#%flag
#% key: s
#% description: Excessive use of segment library
#% guisection: flags
#%end

#%flag
#% key: t
#% description: Truncation of ellipsoids
#% guisection: flags
#%end

#%flag
#% key: v
#% description: empirical confirmation and visualization
#% guisection: flags
#%end

#%flag
#% key: z
#% description: Test mode for multi-core processing (results from tiles are not put together)
#% guisection: flags
#%end

#%option
#% key: prefix
#% type: string
#% description: Prefix for output files and folders
#% required: yes
#% multiple: no
#%end

#%option
#% key: cellsize
#% type: string
#% description: Pixel size (-c, -i, -l)
#% required: no
#% multiple: no
#%end

#%option
#% key: adm
#% type: string
#% description: Area of detail map (N, S, W, E; -v)
#% required: no
#% multiple: yes
#%end

#%option
#% key: elldens
#% type: string
#% description: Ellipsoid density (-c, -l)
#% required: no
#% multiple: no
#%end

#%option
#% key: observed
#% type: string
#% gisprompt: old,raster,cell
#% description: Name of observed landslides raster map (-v)
#% required: no
#% multiple: no
#%end

#%option
#% key: elevation
#% type: string
#% gisprompt: old,raster,dcell
#% description: Name of elevation raster map (-c, -i, -l)
#% required: no
#% multiple: no
#%end

#%option
#% key: soilclass
#% type: string
#% gisprompt: old,raster,cell
#% description: Name of soil class raster map (-c, -i, -l)
#% required: no
#% multiple: no
#%end

#%option
#% key: slopeunits
#% type: string
#% gisprompt: old,raster,cell
#% description: Name of slope units raster map (-v)
#% required: no
#% multiple: no
#%end

#%option
#% key: dxl
#% type: string
#% gisprompt: old,raster,cell
#% description: Name of average dx slope of layers raster map (-l)
#% required: no
#% multiple: no
#%end

#%option
#% key: dyl
#% type: string
#% gisprompt: old,raster,cell
#% description: Name of average dy slope of layers raster map (-l)
#% required: no
#% multiple: no
#%end

#%option
#% key: numlayers
#% type: string
#% description: Number of layers for each soil class, comma separated (-c, -i)
#% required: no
#% multiple: yes
#%end

#%option
#% key: depthmaps
#% type: string
#% gisprompt: old,raster,cell
#% description: Names of depth of layer bottom raster maps, comma separated (-c, -i)
#% required: no 
#% multiple: yes
#%end

#%option
#% key: depthvals
#% type: string
#% description: Depth of layer bottom values, comma separated (-c, -i)
#% required: no
#% multiple: yes
#%end

#%option
#% key: depthstats
#% type: string
#% description: Statistical properties of top layer depth (-p)
#% required: no
#% multiple: yes
#%end

#%option
#% key: prelayers
#% type: string
#% description: Prefix for depth of layer bottom raster maps (-l)
#% required: no
#% multiple: no
#%end

#%option
#% key: classlayers
#% type: string
#% description: Soil class to be assigned to each layer, comma separated (-l)
#% required: no
#% multiple: yes
#%end

#%option
#% key: maxlayers
#% type: string
#% description: Maximum number of layers relevant for one ellipsoid (-l)
#% required: no
#% multiple: no
#%end

#%option
#% key: geotech
#% type: string
#% description: Geotechnical parameters for each soil class and/or layer, comma separated (-c, -i, -l)
#% required: no
#% multiple: yes
#%end

#%option
#% key: seepage
#% type: string
#% description: Seepage direction (-c, -i, -l)
#% required: no
#% multiple: no
#%end

#%option
#% key: ellips
#% type: string
#% description: Ellipsoid parameters, comma separated (-c, -l)
#% required: no
#% multiple: yes
#%end

#%option
#% key: sampling
#% type: string
#% description: Sampling parameters, comma separated (-p)
#% required: no
#% multiple: yes
#%end

#%option
#% key: segsize
#% type: string
#% description: Size of one segment in rows or columns (-c, -i, -l)
#% required: no
#% multiple: no
#%end

#%option
#% key: nsegs
#% type: string
#% description: Number of segments to be held in memory (-c, -i, -l)
#% required: no
#% multiple: no
#%end

#%option
#% key: tilesx
#% type: string
#% description: Number of tiles in x direction (-m)
#% required: no
#% multiple: no
#%end

#%option
#% key: tilesy
#% type: string
#% description: Number of tiles in y direction (-m)
#% required: no
#% multiple: no
#%end

#%option
#% key: overlap
#% type: string
#% description: Overlap of tiles (-m, -v)
#% required: no
#% multiple: no
#%end

#%option
#% key: ncores
#% type: string
#% description: Number of cores to use (-m)
#% required: no
#% multiple: no
#%end


#Importing libraries

import grass.script as grass
from grass.script import core as grasscore
import os
import Queue
import re
import shutil
import subprocess
import sys
import threading
import time


#Defining fundamental variables

exitFlag=0

queueLock=threading.Lock()
workQueue=Queue.Queue()

ambvars=grass.gisenv() #path to grass data base
locpath=ambvars.GISDBASE+'/'+ambvars.LOCATION_NAME #path to grass location
mapsetpath=ambvars.GISDBASE+ '/'+ambvars.LOCATION_NAME+'/'+ambvars.MAPSET #path to grass mapset directory
temppath=ambvars.GISDBASE+'/'+ambvars.LOCATION_NAME+'/'+ambvars.MAPSET+'/.tmp/rtemp' #path to temporary directory
scriptpath=os.environ['GISBASE']+'/scripts' #path to grass scripts


#Function for paths

def which(file):
    for path in os.environ["PATH"].split(':'):
        if os.path.exists(path+'/'+file):
            return path+'/'+file
    return None


if not which('grass79') == None: bingrass=which('grass79')
elif not which('grass78') == None: bingrass=which('grass78')
elif not which('grass77') == None: bingrass=which('grass77')
elif not which('grass76') == None: bingrass=which('grass76')
elif not which('grass75') == None: bingrass=which('grass75')
elif not which('grass74') == None: bingrass=which('grass74')
elif not which('grass73') == None: bingrass=which('grass73')
elif not which('grass72') == None: bingrass=which('grass72')
elif not which('grass71') == None: bingrass=which('grass71')
elif not which('grass70') == None: bingrass=which('grass70') #reading grass binary


#Function for error message

def ErrorMessage():
    grass.message(' ')
    grass.error('Please specify all necessary parameters.')
    grass.message(' ')
    sys.exit()


#Class for threading

class myThread (threading.Thread):
    def __init__(self, name, q):
        threading.Thread.__init__(self)
        self.name=name
        self.q=q
    def run(self):
        process_data(self.name, self.q)


#Function for start of multi-core processing

def StartBatch(jid):
    print 'Processing tile no. %s' %jid
    esegui='bash '+temppath+'/tmp'+str(jid)+'/batch'+str(jid)
    os.system(esegui+' < /dev/null > '+temppath+'/out'+str(jid))
    print 'Tile no. %s completed.' %jid
    return


#Function supporting start of multi-core processing

def process_data(threadName, q):
    while not exitFlag:
        queueLock.acquire()
        if not workQueue.empty():
            time.sleep(0.25)
            data=q.get()
            queueLock.release()
            StartBatch(data)
        else:
            queueLock.release()


#Calling main function

def main():


#Setting flags and options

    aflag=flags['a']
    cflag=flags['c']
    dflag=flags['d']
    iflag=flags['i']
    lflag=flags['l']
    mflag=flags['m']
    pflag=flags['p']
    sflag=flags['s']
    tflag=flags['t']
    vflag=flags['v']
    zflag=flags['z']
    pf=options['prefix']
    cellsize=options['cellsize']
    adm=options['adm']
    elldens=options['elldens']
    observed=options['observed']
    elevation=options['elevation']
    soilclass=options['soilclass']
    slopeunits=options['slopeunits']
    dxl=options['dxl']
    dyl=options['dyl']
    numlayers=options['numlayers']
    depthmaps=options['depthmaps']
    depthvals=options['depthvals']
    depthstats=options['depthstats']
    prelayers=options['prelayers']
    classlayers=options['classlayers']
    maxlayers=options['maxlayers']
    geotech=options['geotech']
    seepage=options['seepage']
    ellips=options['ellips']
    sampling=options['sampling']
    segsize=options['segsize']
    nsegs=options['nsegs']
    tilesx0=options['tilesx']
    tilesy0=options['tilesy']
    overlap0=options['overlap']
    ncores0=options['ncores']


#Making sure that all necessary parameters were specified

    if ( cflag and iflag ) or ( cflag and lflag ) or (iflag and lflag ):
        grass.message(' ')
        grass.error('Please specify only one mode.')
        grass.message(' ')
        sys.exit()

    if elldens == '0': #no multi-core processing or slope failure probability, but documentation if only one ellipsoid is tested
        dflag = 1
        mflag = None
        pflag = None
    if iflag: #no additional output or documentation for infinite slope stability mode
        aflag = None
        dflag = None
        tflag = None
    if pflag: #no documentation with slope failure probability
        dflag = None
    if zflag: #no documentation or confirmation and visualization for test mode
        dflag = None
        vflag = None

    if not pf:
        ErrorMessage()
    if cflag or iflag or lflag: #slope stability model
        if not elevation or not soilclass or not geotech or not seepage or not cellsize: ErrorMessage()
    if cflag or lflag:  #ellipsoid modes
        if not ellips or not elldens: ErrorMessage()
    if cflag or iflag: #soil class mode
        if not numlayers or ( not depthmaps and not depthvals ): ErrorMessage()
    if lflag: #layer mode
        if not prelayers or not classlayers: ErrorMessage()
    if mflag:  #multi-core processing
        if not tilesx0 or not tilesy0 or not overlap0 or not ncores0: ErrorMessage()
    if pflag: #slope failure probability
        if not sampling: ErrorMessage()

    if not maxlayers: maxlayers=-9999 #if maximum number of layers for one ellipsoid is not defined, setting to no data
    if not segsize: segsize=16 #if size of segments is not defined, setting default value
    if not nsegs: nsegs=16 #if number of of segments to be held in memory is not defined, setting default value
    vonly=0 #setting control for confirmation mode only to 0 (negative, initial value)

    d_numlayers=2 #setting initial value for number of layers

    print
    print
    print '##############################################################################'
    print '#'
    print '# MODULE:       r.slope.stability.py'
    print '#'
    print '# AUTHOR:       Martin Mergili'
    print '# CONTRIBUTORS: Massimiliano Alvioli, Ivan Marchesini'
    print '#'
    print '# PURPOSE:      The slope stability model'
    print '#'
    print '# COPYRIGHT:    (c) 2009 - 2017 by the author'
    print '#               (c) 2009 - 2017 by the BOKU University, Vienna'
    print '#               (c) 2015 - 2017 by the University of Vienna'
    print '#               (c) 2013 - 2017 by the CNR-IRPI Perugia'
    print '#               (c) 2000 - 2017 by the GRASS Development Team'
    print '#'
    print '# VERSION:      20170406 (6 April 2017)'
    print '#'
    print '# PREFIX:       %s' %pf
    print '#'
    print '#               This program is free software under the GNU General Public'
    print '#               License (>=v2). Read the file COPYING that comes with GRASS'
    print '#               for details.'
    print '#'
    print '##############################################################################'
    print
    print
    print '1. PREPARING ENVIRONMENT.'


#Preparing environment

    os.system('rm -rf '+temppath) #if necessary, deleting temporary directory
    os.system('mkdir '+temppath) #creating temporary directory

    if pflag: length_all=12 #length of list with geotechnical parameters per layer and soil class
    else: length_all=6

    if mflag: #converting parameters for multi-core processing to numbers:
        tilesx=int(tilesx0)
        tilesy=int(tilesy0)
        ncores=int(ncores0)

    if overlap0: overlap=float(overlap0) #converting overlap string to number

    if cflag or iflag: #for soil class mode or infinite slope stability mode:

        if depthmaps: #if layer depths are defined by maps:
            depthmaps=map(str, depthmaps.split(',')) #splitting input
            j=0 #initializing counter

            for i in depthmaps: #loop over all parts of input:
                j=j+1 #updating counter
                grass.run_command('g.copy', rast=(i,'xdepth'+str(j)), overwrite=True, quiet=True) #temporary layer depth maps

    if cflag or iflag or lflag: #for slope stability model:
        p1file=open(temppath + '/param1.txt', 'w') #opening parameter files for writing
        p2file=open(temppath + '/param2.txt', 'w')
        p3file=open(temppath + '/param3.txt', 'w')
        p4file=open(temppath + '/cellsize.txt', 'w')


    #Writing param2.txt file

        print>> p2file, 'param2' #writing header to file

        if ( cflag or iflag ): #for soil class or infinite slope stability mode:
            numlayers=map(str, numlayers.split(',')) #splitting input
            imaxxx=0
            for i in numlayers:
                print>> p2file, i #writing number of layers in each soil class to file
                if int(i)>imaxxx: imaxxx=int(i)

        else: #for layer mode:
            geotech_gamma=['99999'] #initializing lists of geotechnical parameters
            geotech_c=['99999']
            geotech_phi=['89']
            geotech_theta=['0']
            geotech_cstdev=['0']
            geotech_cmin=['99999']
            geotech_cmax=['99999']
            geotech_phistdev=['0']
            geotech_phimin=['89']
            geotech_phimax=['89']
            geotech=map(str, geotech.split(',')) #splitting input
            numcat=(len(geotech)+1)/(length_all-1) #number of soil classes
            j=0 #initializing counter
            for i in geotech: #loop over all parts of input:
                j=j+1 #updating counter
                if j==length_all: j=1 #if end of class is reached, go to next class
                if j==2: geotech_gamma.append(i) #soil specific weight
                if j==3: geotech_c.append(i) #cohesion
                if j==4: geotech_phi.append(i) #friction angle
                if j==5: geotech_theta.append(i) #saturated water content
                if pflag:
                    if j==6: geotech_cstdev.append(i) #cohesion, standard deviation
                    if j==7: geotech_cmin.append(i) #cohesion, minimum
                    if j==8: geotech_cmax.append(i) #cohesion, maximum
                    if j==9: geotech_phistdev.append(i) #angle of internal friction, standard deviation
                    if j==10: geotech_phimin.append(i) #angle of internal friction, minimum
                    if j==11: geotech_phimax.append(i) #angle of internal friction, maximum
                if j>1: print>> p2file, i #writing parameters to file


    #Writing param3.txt file

        print>> p3file, 'param3' #writing header to file

        if ( cflag or iflag ): #for soil class or infinite slope stability mode:
            if depthvals: depthvals=map(str, depthvals.split(',')) #splitting input 
            geotech=map(str, geotech.split(',')) #splitting input
            length=len(geotech)+1 #reading length of dataset
            k=0 #initializing counter for geotechnical parameters 
            n=0 #initializing counter for layer bottom depth values
            for i in geotech: #loop over geotechnical parameter and layer bottom depth values: 
                if k not in range(0,length,length_all) and k not in range(1,length,length_all): #for geotechnical parameter values:
                    print>> p3file, i #writing parameter values to file
                k=k+1 #increasing counter for geotechnical parameters
                if k in range(length_all,length,length_all) : #for layer bottom depth values:
                    if depthmaps: print>> p3file, -9999 #if depth map is provided, writing no data value to file
                    else:               
                        print>> p3file, depthvals[n] #else, writing depth of layer bottom to file
                        n=n+1 #increasing counter for layers

        else: #for layer mode:
            classlayers=map(str, classlayers.split(',')) #splitting input
            j=0 #initializing counter
            for i in classlayers: #loop over all parts of input:
                j=j+1 #updating counter
                print>> p3file, geotech_gamma[int(i)] #writing soil specific weight for layer to file
                print>> p3file, geotech_c[int(i)] #writing cohesion for layer to file
                print>> p3file, geotech_phi[int(i)] #writing friction angle for layer to file
                print>> p3file, geotech_theta[int(i)] #writing saturated water content for layer to file

                if pflag:
                    print>> p3file, geotech_cstdev[int(i)] #writing cohesion, standard deviation for layer to file
                    print>> p3file, geotech_cmin[int(i)] #writing cohesion, minimum for layer to file
                    print>> p3file, geotech_cmax[int(i)] #writing cohesion, maximum for layer to file
                    print>> p3file, geotech_phistdev[int(i)] #writing friction angle, standard deviation for layer to file
                    print>> p3file, geotech_phimin[int(i)] #writing friction angle, minimum for layer to file
                    print>> p3file, geotech_phimax[int(i)] #writing friction angle, maximum for layer to file

                grass.run_command('g.copy', rast=(prelayers+str(j),'xdepth'+str(j)), overwrite=True, quiet=True) #temporary layer depth maps
            d_numlayers=j #number of layers


    #Writing param1.txt file

        print>> p1file, 'param1' #writing header to file
        if cflag or iflag: maxcat=int(grass.raster_info(soilclass)['max']) #reading number of soil classes
        print>> p1file, temppath #writing temporary directory to file
        print>> p1file, ambvars.MAPSET #writing mapset to file
        if cflag: print>> p1file, '1' #writing control for soil class mode to file
        else: print>> p1file, '0'
        if iflag: print>> p1file, '1' #writing control for infinite slope stability mode to file
        else: print>> p1file, '0'
        if lflag: print>> p1file, '1' #writing control for layer mode to file
        else: print>> p1file, '0'
        if tflag: print>> p1file, '1' #writing control for mode with truncated ellipsoids to file
        else: print>> p1file, '0'
        if aflag: print>> p1file, '1' #writing control for writing additional output
        else: print>> p1file, '0'
        print>> p1file, seepage #writing control for seepage direction to file
        if dflag: print>> p1file, '1' #writing control for documentation to file
        else: print>> p1file, '0'
 
        if cflag or iflag:
            print>> p1file, maxcat #writing number of soil classes to file
            print>> p1file, int(imaxxx) #writing temporary value for number of layers to file
                #BE CAREFUL THIS IS TEMPORARY AND MAY CAUSE AN ERROR!!!!!!!!!!!!!!!!!!!!!!

        else:
            print>> p1file, numcat #writing number of soil classes to file
            print>> p1file, j #writing number of layers to file
            print>> p1file, maxlayers #writing maximum number of layers for one ellipsoid to file

        if sflag: print>> p1file, '1' #writing control for excessive use of segment library to file
        else: print>> p1file, '0'
        if pflag: print>> p1file, '1' #writing control for slope failure probability to file
        else: print>> p1file, '0'

        if pflag:
            sampling=map(str, sampling.split(',')) #splitting input
            for isteps in sampling: print>> p1file, isteps #writing sampling parameters to file

        if ( cflag or iflag ) and pflag and ( tflag or iflag ):
            if int(sampling[3])>1:
                depthstats=map(str, depthstats.split(',')) #splitting input
                for istats in depthstats: print>> p1file, istats #writing statistical properties of top layer to file

        print>> p1file, segsize #writing segment size to file
        print>> p1file, nsegs #writing number of segments to file


    #Completing param3.txt file

        if cflag or lflag: #for modes with ellipsoids:
            ellpar=map(str, ellips.split(',')) #ellipsoid parameters
            for i in ellpar: print>> p3file, i #writing ellipsoid parameters to file  
            print>> p3file, elldens #writing ellipsoid density to file

        print>> p4file, cellsize #writing pixel size to file

        p1file.close() #closing parameter files
        p2file.close()   
        p3file.close()
        p4file.close()


#Setting and storing region

    if cflag or iflag or lflag: grass.run_command('g.region', res=cellsize) #updating pixel size

    if mflag or vflag: #if mode involves extent change, storing values

        region = grasscore.region() #reading region of study area
        e0 = region['e'] #reading E boundary of study area
        w0 = region['w'] #reading W boundary of study area
        n0 = region['n'] #reading N boundary of study area
        s0 = region['s'] #reading S boundary of study area   


#Preparing temporary raster maps for slope stability model

    if cflag or iflag or lflag: #for modes with slope stability model:

        grass.run_command('g.copy', rast=(elevation,'xelev'), overwrite=True, quiet=True) #temporary elevation map
        grass.run_command('g.copy', rast=(soilclass,'xsoilclass'), overwrite=True, quiet=True) #temporary soil class map

    if lflag and seepage == '2': #for layer mode with layer-parallel seepage:

        print '%i soil classes and %i layers.' %(numcat, j)

        if dxl and dyl: #if average slope of layers is given, reading input rasters:

            grass.run_command('g.copy', rast=(dxl,'xdzdyl'), overwrite=True, quiet=True) #average slope in x direction
            grass.run_command('g.copy', rast=(dyl,'xdzdxl'), overwrite=True, quiet=True) #average slope in y direction

        else: #if not given, computing average slope of layers:
            grass.run_command('r.slope.aspect', elevation='xelev', dx='xdx0', dy='xdy0', quiet=True, overwrite=True)
            grass.mapcalc('"sumdxl"=0', overwrite=True, quiet=True)
            grass.mapcalc('"sumdyl"=0', overwrite=True, quiet=True)
            grass.mapcalc('"numdxl"=0', overwrite=True, quiet=True)
            grass.mapcalc('"numdyl"=0', overwrite=True, quiet=True)

            for m in range(1,j+1):
                grass.mapcalc('"xelevl"=if("xdepth%s">0,"xelev"-"xdepth%s",1/0)' %(m,m), overwrite=True, quiet=True)
                grass.run_command('r.slope.aspect', elevation='xelevl', dx='dxl', dy='dyl', quiet=True, overwrite=True)
                grass.mapcalc('"dxtestl"=isnull("dxl")', overwrite=True, quiet=True)
                grass.mapcalc('"dytestl"=isnull("dyl")', overwrite=True, quiet=True)
                grass.mapcalc('"sumdxl"=if("dxtestl"==0,"sumdxl"+"dxl","sumdxl")', overwrite=True, quiet=True)
                grass.mapcalc('"sumdyl"=if("dytestl"==0,"sumdyl"+"dyl","sumdyl")', overwrite=True, quiet=True)
                grass.mapcalc('"numdxl"=if("dxtestl"==0,"numdxl"+1,"numdxl")', overwrite=True, quiet=True)
                grass.mapcalc('"numdyl"=if("dytestl"==0,"numdyl"+1,"numdyl")', overwrite=True, quiet=True)
                grass.run_command('g.remove', flags='f', type='rast', name='dxl,dyl', quiet=True)

                sys.stdout.write('Computing slope for layer %s of %s ...\r' %(m, j))
                sys.stdout.flush()

            grass.mapcalc('"xdxl0"=if("numdxl"==0,"xdx0","sumdxl"/"numdxl")', overwrite=True, quiet=True)
            grass.mapcalc('"xdyl0"=if("numdyl"==0,"xdy0","sumdyl"/"numdyl")', overwrite=True, quiet=True)
            grass.mapcalc('xdzdx="xdy0"', overwrite=True, quiet=True)
            grass.mapcalc('xdzdy="xdx0"', overwrite=True, quiet=True)
            grass.mapcalc('xdzdxl="xdyl0"', overwrite=True, quiet=True)
            grass.mapcalc('xdzdyl="xdxl0"', overwrite=True, quiet=True)
            grass.run_command('g.remove', flags='f', type='rast', name='xdx0,xdy0,xdxl0,xdyl0,xelevl,dxtestl,dytestl,sumdxl,sumdyl,numdxl,numdyl', quiet=True)

            grass.run_command('g.copy', rast=('xdzdxl','%s_dyl' %pf), overwrite=True, quiet=True) #average slope in x direction
            grass.run_command('g.copy', rast=('xdzdyl','%s_dxl' %pf), overwrite=True, quiet=True) #average slope in y direction

            print 'Computing slopes ... completed.                  '

        print

    else: #if average slope of layers is not required, setting to no data value

        grass.mapcalc('"xdzdxl"=-9999', overwrite=True, quiet=True)
        grass.mapcalc('"xdzdyl"=-9999', overwrite=True, quiet=True)

    if cflag or iflag or lflag: #mode with slope stability model:

        print '2. EVALUATING SLOPE STABILITY.'

        start = time.time() #storing time (start of main computation)


    #Single-core processing

        if not mflag: #if the study area is computed as one entity:

            os.environ['XINT']='1' #exporting id
            grass.run_command('r.slope.stability.main', flags='1') #executing slope stability model

            if aflag: grass.mapcalc('"r_sindex"=float(round(10000*"r_cumfos_crit"/"r_cumsurf"))/10000', overwrite=True, quiet=True)
                #computing susceptibility index


        #Making result raster maps permanent

            if iflag or not ( elldens == '0' ): #for ellipsoid modes with more than one ellipsoid or infinite slope stability mode:
                grass.run_command('g.rename', rast=('r_fos','%s_fos' %pf), overwrite=True, quiet=True) #factor of safety (fos)
                if pflag:
                    grass.run_command('g.rename', rast=('r_pfail','%s_pfail' %pf), overwrite=True, quiet=True) #slope failure probability
                    grass.run_command('g.rename', rast=('r_fos_stdev','%s_fos_stdev' %pf), overwrite=True, quiet=True) #stdev of fos

            if ( cflag or lflag ) and not ( elldens == '0' ) and aflag:
                #for ellipsoid modes with more than one ellipsoid and additional output:
                grass.run_command('g.rename', rast=('r_sindex','%s_sindex' %pf), overwrite=True, quiet=True) #susceptibility index
                grass.run_command('g.rename', rast=('r_lcrit','%s_lcrit' %pf), overwrite=True, quiet=True) #length of critical ellipsoid
                grass.run_command('g.rename', rast=('r_wcrit','%s_wcrit' %pf), overwrite=True, quiet=True) #width of critical ellipsoid
                grass.run_command('g.rename', rast=('r_zbcrit','%s_zbcrit' %pf), overwrite=True, quiet=True) #zb of critical ellipsoid

            if aflag: #for mode with additional output:
                grass.run_command('g.rename', rast=('r_depth','%s_depth' %pf), overwrite=True, quiet=True) #depth of critical slip surface
            
            if lflag and not elldens == '0' and aflag: #for layer mode with more than one ellipsoid and additional output:
                grass.run_command('g.rename', rast=('r_layer','%s_layer' %pf), overwrite=True, quiet=True) #id of critical layer

            if elldens == '0': #for mode with one ellipsoid only:
                grass.run_command('g.rename', rast=('r_depth','%s_depth' %pf), overwrite=True, quiet=True) #depth of critical slip surface


    #Multi-core processing

        else: #if the study area is split into a defined number of tiles:


        #Creating tiles

            ntiles=tilesy*tilesx #number of tiles
            dx=region['e']-region['w'] #E-W-extent of study area
            dy=region['n']-region['s'] #N-S-extent of study area
            tdx=(dx+overlap*(tilesx-1))/tilesx #E-W-extent of 1 tile
            tdy=(dy+overlap*(tilesy-1))/tilesy #N-S-extent of 1 tile

            print
            print 'Splitting study area into %s tiles of %s x %s m.' %(ntiles,tdx,tdy)

            jid=0 #resetting counter for tiles

            for nx in range(1,tilesx+1): #starting loop over all tiles in E-W-direction:
                for ny in range(1,tilesy+1): #starting loop over all tiles in N-S-direction:

                    jid=jid+1 #updating counter for tiles

                    tx1=w0+(nx-1)*(tdx-overlap) #W boundary of tile
                    ty1=s0+(ny-1)*(tdy-overlap) #S boundary of tile
                    tx2=tx1+tdx #E boundary of tile
                    ty2=ty1+tdy #N boundary of tile

                    tx1=float(cellsize)*round(tx1/float(cellsize),0) #adjusting W boundary of tile to pixel size
                    ty1=float(cellsize)*round(ty1/float(cellsize),0) #adjusting S boundary of tile to pixel size
                    tx2=float(cellsize)*round(tx2/float(cellsize),0) #adjusting E boundary of tile to pixel size
                    ty2=float(cellsize)*round(ty2/float(cellsize),0) #adjusting N boundary of tile to pixel size

                    grass.run_command('g.region', n=ty2, s=ty1, w=tx1, e=tx2) #setting region to extent of tile

                    print
                    print 'Writing batch file for tile %s.' %jid
                    print
                    print 'Extent adjusted to pixel size:'
		    print 'West:  %s m' %tx1
                    print 'East:  %s m' %tx2
		    print 'South: %s m' %ty1
		    print 'North: %s m' %ty2
                    print


                #Creating batch file

                    os.mkdir(temppath+'/tmp%s' % jid) #creating directory for batch file
                    strtmp=temppath+'/tmp%s/batch' % jid #file name for batch file
                    out=file(strtmp+str(jid),'w') #creating batch file
                    os.system('rm -rf '+locpath+'/map%s' %jid) #removing old mapset for tile
                    grass.run_command('g.mapset', flags='c', mapset='map%s' %jid) #creating new mapset for tile
                    grass.run_command('g.mapsets', mapset=ambvars.MAPSET, operation='add') #making original mapset active

                    grass.run_command("g.mapset", mapset=ambvars.MAPSET) #switching back to original mapset
                    os.system('mkdir '+locpath+'/map%s/.tmp/rtemp' %jid) #creating directory in mapset for tile
                    os.environ['PATH']+=os.pathsep+os.path.join(temppath+'/tmp%s') %jid #adding path to batch file

                    if aflag: aflag_exp=1 #exporting controls to r.slope.stability.mult.py
                    else: aflag_exp=0
                    if pflag: pflag_exp=1
                    else: pflag_exp=0

                    print >> out,"""#!/bin/bash
export jid=%s
export ntiles=%s
export SHELL=\"/bin/bash\"
export west=%s
export sout=%s
export east=%s
export nort=%s
export cellsize=%s
export aflag=%s
export pflag=%s
export rtemp=%s
export GRASS_BATCH_JOB=%s/r.slope.stability.mult
rm -f ~/.grassrc6
cp %s/*.txt %s/map%s/.tmp/rtemp/
%s -text %s/map%s
unset GRASS_BATCH_JOB""" % (jid,ntiles,tx1,ty1,tx2,ty2,cellsize,aflag_exp,pflag_exp,temppath,scriptpath,temppath,locpath,jid,bingrass,locpath,jid) #creating batch file

                    out.close() #closing batch file
                    fd=os.open(strtmp+str(jid),os.O_RDONLY) #opening batch file
                    os.fchmod(fd,0755) #making batch file executable
                    os.close( fd ) #closing batch file


        #Executing batch processing

            start_batch = time.time() #storing time (start of multi-core processing)

            neff=min(ncores,ntiles)
            threadList=range(1,neff+1)
            nameList=range(1,ntiles+1)
            threads=[]
            for tName in threadList:
                thread=myThread(tName, workQueue)
                thread.start()
                threads.append(thread)
            queueLock.acquire()
            for word in nameList:
                workQueue.put(word)
            queueLock.release()
            while not workQueue.empty():
               pass
            global exitFlag
            exitFlag=1
            for t in threads:
                t.join()
            print
            print 'Batch processing completed.'
            print

            stop_batch = time.time() #storing time (stop of multi-core processing)
            comptime_batch = stop_batch - start_batch #storing computing time for batch processing in seconds


        #Collecting results from all tiles

            if cflag or lflag: lbtmmaxmax = 0 #initializing overall maximum number of layers for one ellipsoid
            for jid in nameList: #loop over all tiles:
                if not zflag: #if mode is not test mode:

                    if iflag or not ( elldens=='0'):
                        #for ellipsoid modes with more than one ellipsoid or infinite slope stability mode:
                        grass.run_command('r.in.gdal', input=temppath+'/r_fos'+str(jid),
                            output='r_fos'+str(jid), overwrite=True, quiet=True) #factor of safety (fos)
                        if pflag:
                            grass.run_command('r.in.gdal', input=temppath+'/r_pfail'+str(jid),
                                output='r_pfail'+str(jid), overwrite=True, quiet=True) #slope failure probability
                            grass.run_command('r.in.gdal', input=temppath+'/r_fos_stdev'+str(jid),
                                output='r_fos_stdev'+str(jid), overwrite=True, quiet=True) #standard deviation of fos

                    if ( cflag or lflag ) and not ( elldens=='0') and aflag:
                        #for ellipsoid modes with more than one ellipsoid and additional output:
                        grass.run_command('r.in.gdal', input=temppath+'/r_sindex'+str(jid),
                            output='r_sindex'+str(jid), overwrite=True, quiet=True) #susceptibility index
                        grass.run_command('r.in.gdal', input=temppath+'/r_lcrit'+str(jid),
                            output='r_lcrit'+str(jid), overwrite=True, quiet=True) #length of critical ellipsoid
                        grass.run_command('r.in.gdal', input=temppath+'/r_wcrit'+str(jid),
                            output='r_wcrit'+str(jid), overwrite=True, quiet=True) #width of critical ellipsoid
                        grass.run_command('r.in.gdal', input=temppath+'/r_zbcrit'+str(jid),
                            output='r_zbcrit'+str(jid), overwrite=True, quiet=True) #zb of critical ellipsoid

                    if aflag: #for mode with additional output
                        grass.run_command('r.in.gdal', input=temppath+'/r_depth'+str(jid),
                            output='r_depth'+str(jid), overwrite=True, quiet=True) #depth of critical slip surface

                    if lflag and not ( elldens=='0' ) and aflag: #for layer mode with more than one ellipsoid and additional output:
                        grass.run_command('r.in.gdal', input=temppath+'/r_layer'+str(jid),
                            output='r_layer'+str(jid), overwrite=True, quiet=True) #id of critical layer

                if ( cflag or lflag ): #for ellipsoid modes:

                    sumfile=open(temppath+'/summary3'+str(jid)+'.txt', 'r')
                        #opening summary file for reading
                    lbtmmax=sumfile.readline() #reading maximum number of layers for one ellipsoid
                    lbtmmax=int(lbtmmax.replace('\n', ''))
                    lbtmalw=sumfile.readline() #reading maximum number of layers allowed
                    lbtmalw=int(lbtmalw.replace('\n', ''))
                    sumfile.close() #closing file
                    if lbtmmax > lbtmmaxmax: lbtmmaxmax = lbtmmax #updating maximum number of layers for one ellipsoid, if necessary

            grass.run_command('g.region', n=n0, s=s0, w=w0, e=e0, quiet=True) #restoring original extent


        #Initializing result maps

            if not zflag: #if mode is not test mode:

                if iflag or not ( elldens=='0'): #for ellipsoid modes with more than one ellipsoid or infinite slope stability mode:
                    grass.mapcalc('"%s_fos"=9999' %pf, overwrite=True, quiet=True)
                    if pflag:
                        grass.mapcalc('"%s_pfail"=-9999' %pf, overwrite=True, quiet=True)
                        grass.mapcalc('"%s_fos_stdev"=-9999' %pf, overwrite=True, quiet=True)

                if ( cflag or lflag ) and not ( elldens=='0') and aflag:
                    #for ellipsoid modes with more than one ellipsoid and additional output:
                    grass.mapcalc('"%s_sindex"=-9999' %pf, overwrite=True, quiet=True)
                    grass.mapcalc('"%s_lcrit"=-9999' %pf, overwrite=True, quiet=True)
                    grass.mapcalc('"%s_wcrit"=-9999' %pf, overwrite=True, quiet=True)
                    grass.mapcalc('"%s_zbcrit"=-9999' %pf, overwrite=True, quiet=True)

                if aflag: grass.mapcalc('"%s_depth"=-9999' %pf, overwrite=True, quiet=True) #for mode with additional output
                if lflag and not ( elldens=='0' ) and aflag: #for layer mode with more than one ellipsoid:
                    grass.mapcalc('"%s_layer"=-9999' %pf, overwrite=True, quiet=True)


            #Superimposing results from all tiles

                for jid in range(1,ntiles+1): #loop over all tiles:

                    if iflag or not ( elldens=='0'):
                        #for ellipsoid modes with more than one ellipsoid or infinite slope stability mode:

                        grass.mapcalc('"%s_fos"=if(isnull("r_fos%s")==1,"%s_fos",if("r_fos%s">"%s_fos","%s_fos","r_fos%s"))'
                            %(pf,str(jid),pf,str(jid), pf,pf,str(jid)), overwrite=True, quiet=True)
                        if pflag:
                            grass.mapcalc('"%s_pfail"=if(isnull("r_pfail%s")==1,"%s_pfail",if("r_pfail%s"<"%s_pfail","%s_pfail","r_pfail%s"))'
                                %(pf,str(jid),pf,str(jid), pf,pf,str(jid)), overwrite=True, quiet=True)
                            grass.mapcalc('"%s_fos_stdev"=if(isnull("r_fos_stdev%s")==1,"%s_fos_stdev",if("r_fos%s">"%s_fos","%s_fos_stdev","r_fos_stdev%s"))'
                                %(pf,str(jid),pf,str(jid), pf,pf,str(jid)), overwrite=True, quiet=True)

                    if ( cflag or lflag ) and not ( elldens=='0') and aflag:
                        #for ellipsoid modes with more than one ellipsoid and additional output:
                        grass.mapcalc('"%s_sindex"=if(isnull("r_sindex%s")==1,"%s_sindex",if("r_sindex%s"<"%s_sindex","%s_sindex","r_sindex%s"))'
                            %(pf,str(jid),pf,str(jid), pf,pf,str(jid)), overwrite=True, quiet=True)
                        grass.mapcalc('"%s_lcrit"=if(isnull("r_lcrit%s")==1,"%s_lcrit",if("r_fos%s">"%s_fos","%s_lcrit","r_lcrit%s"))'
                            %(pf,str(jid),pf,str(jid), pf,pf,str(jid)), overwrite=True, quiet=True)
                        grass.mapcalc('"%s_wcrit"=if(isnull("r_wcrit%s")==1,"%s_wcrit",if("r_fos%s">"%s_fos","%s_wcrit","r_wcrit%s"))'
                            %(pf,str(jid),pf,str(jid), pf,pf,str(jid)), overwrite=True, quiet=True)
                        grass.mapcalc('"%s_zbcrit"=if(isnull("r_zbcrit%s")==1,"%s_zbcrit",if("r_fos%s">"%s_fos","%s_zbcrit","r_zbcrit%s"))'
                            %(pf,str(jid),pf,str(jid), pf,pf,str(jid)), overwrite=True, quiet=True)

                    if aflag: grass.mapcalc('"%s_depth"=if(isnull("r_depth%s")==1,"%s_depth",if("r_fos%s">"%s_fos","%s_depth","r_depth%s"))'
                        %(pf,str(jid),pf,str(jid), pf,pf,str(jid)), overwrite=True, quiet=True) #for mode with additional output

                    if lflag and not ( elldens=='0' ) and aflag: #for layer mode with more than one ellipsoid and additional output:
                        grass.mapcalc('"%s_layer"=if(isnull("r_layer%s")==1,"%s_layer",if("r_fos%s">"%s_fos","%s_layer","r_layer%s"))'
                            %(pf,str(jid),pf,str(jid),pf,pf,str(jid)), overwrite=True, quiet=True)

                for jid in range(1,ntiles+1): os.system('rm -rf '+locpath+'/map'+str(jid)) #removing mapsets for all tiles

        stop = time.time() #storing time (end of main computation)
        comptime = stop - start #storing computing time in seconds


    #Creating structure for storing results

        if os.path.exists(pf+'_results'): os.system('rm -rf '+pf+'_results') #if result directory already exists, deleting it
        os.mkdir(pf+'_results') #creating directory for results
        os.mkdir(pf+'_results/'+pf+'_files') #creating directory for files
        os.mkdir(pf+'_results/'+pf+'_hmaps') #creating directory for maps with hillshade
        os.mkdir(pf+'_results/'+pf+'_maps') #creating directory for maps
        os.mkdir(pf+'_results/'+pf+'_plots') #creating directory for plots
        os.mkdir(pf+'_results/'+pf+'_tiffs') #creating directory for tiffs


    # Exporting result raster maps to tiff files

        if not zflag: #if mode is not test mode:

            if iflag or not ( elldens=='0' ): #for ellipsoid modes with more than one ellipsoid or infinite slope stability mode:

                grass.run_command('r.out.gdal', input=pf+'_fos', output=pf+'_results/'+pf+'_tiffs/'+pf+'_fos.tif',
                    format='GTiff', flags='c', quiet=True, overwrite=True)
                if pflag:
                    grass.run_command('r.out.gdal', input=pf+'_pfail', output=pf+'_results/'+pf+'_tiffs/'+pf+'_pfail.tif',
                        format='GTiff', flags='c', quiet=True, overwrite=True)
                    grass.run_command('r.out.gdal', input=pf+'_fos_stdev', output=pf+'_results/'+pf+'_tiffs/'+pf+'_fos_stdev.tif',
                        format='GTiff', flags='c', quiet=True, overwrite=True)

            if ( cflag or lflag ) and not ( elldens=='0' ) and aflag:
                #for ellipsoid modes with more than one ellipsoid and additional output:
                grass.run_command('r.out.gdal', input=pf+'_sindex', output=pf+'_results/'+pf+'_tiffs/'+pf+'_sindex.tif',
                    format='GTiff', flags='c', quiet=True, overwrite=True)
                grass.run_command('r.out.gdal', input=pf+'_lcrit', output=pf+'_results/'+pf+'_tiffs/'+pf+'_lcrit.tif',
                    format='GTiff', flags='c', quiet=True, overwrite=True)
                grass.run_command('r.out.gdal', input=pf+'_wcrit', output=pf+'_results/'+pf+'_tiffs/'+pf+'_wcrit.tif',
                    format='GTiff', flags='c', quiet=True, overwrite=True)
                grass.run_command('r.out.gdal', input=pf+'_zbcrit', output=pf+'_results/'+pf+'_tiffs/'+pf+'_zbcrit.tif',
                    format='GTiff', flags='c', quiet=True, overwrite=True)

            if aflag: grass.run_command('r.out.gdal', input=pf+'_depth', output=pf+'_results/'+pf+'_tiffs/'+pf+'_depth.tif',
                format='GTiff', flags='c', quiet=True, overwrite=True) #for mode with additional output

            if lflag and not ( elldens =='0' ) and aflag: #for layer mode with more than one ellipsoid and additional output:
                grass.run_command('r.out.gdal', input=pf+'_layer', output=pf+'_results/'+pf+'_tiffs/'+pf+'_layer.tif',
                    format='GTiff', flags='c', quiet=True, overwrite=True)

            if elldens=='0': #for mode with one ellipsoid:
                grass.run_command('r.out.gdal', input=pf+'_depth', output=pf+'_results/'+pf+'_tiffs/'+pf+'_depth.tif',
                format='GTiff', flags='c', quiet=True, overwrite=True)

            print 'Exporting result maps ... completed.'


    #Writing table for evolution plot

        if ( cflag or lflag ) and not elldens=='0' and not zflag:
            #for ellipsoid-based modes with more than one ellipsoid:

            if mflag: nmaxloop=ntiles+1 #number of loops + 1 for multi-core processing
            else: nmaxloop=2 #number of loops + 1 for single-core processing
            evolfile_linesmin=99999999; #initial value of number of lines of evolution file
            evoltable=open(pf+'_results/'+pf+'_files/'+pf+'_evolution.txt', 'w') #opening evolution table file for writing

            for jid in range (1, nmaxloop): # loop over all tiles
                evolfile=open(temppath+'/evolution'+str(jid)+'.txt', 'r') #opening temporary evolution file
                evolfile_lines = sum(1 for line in evolfile) #reading number of lines
                evolfile.close() #closing file
                evolfile=open(temppath+'/evolution'+str(jid)+'.txt', 'r') #again opening temporary evolution file
                cellallraw=evolfile.readline() #total number of cells
                cellall=float(cellallraw.replace('\n', '')) #removing newline
                evolfile.close() #closing file
                if evolfile_lines>2 and cellall>0 and evolfile_lines<evolfile_linesmin: evolfile_linesmin=evolfile_lines
                    #updating minimum number of lines

            status_max=0.0; #initial value of maximum status related to all pixels
            jidval=0 #initial value for number of valid tiles
            for jid in range (1, nmaxloop): # loop over all tiles

                evolfile=open(temppath+'/evolution'+str(jid)+'.txt', 'r') #opening temporary evolution file
                evolfile_lines = sum(1 for line in evolfile) #reading number of lines
                evolfile.close() #closing file
                evolfile=open(temppath+'/evolution'+str(jid)+'.txt', 'r') #opening temporary evolution file again

                cellallraw=evolfile.readline() #total number of cells
                cellall=float(cellallraw.replace('\n', '')) #removing newline

                if evolfile_lines>2 and cellall>0: #if tile is valid (contains data)
                    jidval=jidval+1 #updating number of valid tiles

                    if jidval==1: #if first valid tile:
                        elldensch=[0] #array for ellipsoid density
                        numsim=[0] #array for number of ellipsoids
                    cellayer_abs=[0] #array for status (number of pixels with fos smaller than 1 or sum of pfail)
                    cellayer=[0] #array for status (ratio of pixels with fos smaller than 1 or sum of pfail)
                    cellchange=[0] #array for change of status
                    cellayer_pre=0 #initializing status of previous step

                    for i in range(1, evolfile_linesmin): #loop over all lines (steps):
                        changeall=evolfile.readline() #reading first line
                        changeall=changeall.replace('\n', '') #removing newline
                        change=map(str, changeall.split('\t')) #splitting input
                        if jidval==1:
                            elldensch.append(change[0]) #appending ellipsoid density to array
                            numsim.append(change[1]) #appending number of tested ellipsoids to array
                        cellayer_abs.append(change[2]) #appending status to array
                        cellayer_ratio=float(change[2])/cellall #status related to all pixels
                        cellayer.append(cellayer_ratio) #appending status related to all pixels to array
                        cellchange_new=float(cellayer_ratio)-float(cellayer_pre) #change of status related to all pixels
                        cellchange.append(cellchange_new) #append change of status related to all pixels to array
                        cellayer_pre=cellayer_ratio #store status related to all pixels for use in next step
                        if cellayer_ratio>status_max: status_max=cellayer_ratio #updating maximum status related to all pixels, if necessary

                    evolfile.close() #closing temporary evolution file

                    if jidval==1: #if first valid tile:
                        for i in range(1, evolfile_linesmin): #loop over all lines:
                            print>> evoltable, '%s\t' %elldensch[i] , #writing ellipsoid density to file
                        print>> evoltable, '\n',
                        for i in range(1, evolfile_linesmin): #loop over all lines:
                            print>> evoltable, '%s\t' %numsim[i] , #writing number of ellipsoids to file
                        print>> evoltable, '\n',
                    for i in range(1, evolfile_linesmin): #loop over all lines:
                        print>> evoltable, '%i\t' %cellall , #writing status to file
                    print>> evoltable, '\n',
                    for i in range(1, evolfile_linesmin): #loop over all lines:
                        print>> evoltable, '%s\t' %cellayer_abs[i] , #writing status to file
                    print>> evoltable, '\n',
                    for i in range(1, evolfile_linesmin): #loop over all lines:
                        print>> evoltable, '%s\t' %cellayer[i] , #writing status related to all pixels to file
                    print>> evoltable, '\n',
                    for i in range(1, evolfile_linesmin): #loop over all lines:
                        print>> evoltable, '%s\t' %cellchange[i] , #writing change of status to file
                    print>> evoltable, '\n',

            evoltable.close() #closing evolution table file


    #Writing table for profile plot

        if ( cflag or lflag ) and dflag: #for ellipsoid modes with documentation:

            if mflag:  #making summary file(s) permanent:
                for jid in range (1, ntiles+1): shutil.copyfile(temppath+'/summary1'+str(jid)+'.txt',
                    pf+'_results/'+pf+'_files/'+pf+'_summary'+str(jid)+'.txt')
            else: shutil.copyfile(temppath+'/summary11.txt',pf+'_results/'+pf+'_files/'+pf+'_summary.txt')
 
            if elldens=='0': #for ellipsoid modes with only one ellipsoid:
                shutil.copyfile(temppath+'/summary21.txt', pf+'_results/'+pf+'_files/'+pf+'_profile.txt')
                    #making profile file permanent
                sumfile=open(temppath+'/summary31.txt', 'r') #opening summary file for reading
                sumfile.readline() #reading first line (not needed here)
                sumfile.readline() #reading second line (not needed here)
                prof_numlayers=sumfile.readline()
                prof_numlayers=int(prof_numlayers.replace('\n', '')) #extracting number of layers
                prof_slipcrit=sumfile.readline()
                prof_slipcrit=int(prof_slipcrit.replace('\n', '')) #extracting id of critical layer
                prof_foscrit=sumfile.readline()
                prof_foscrit=float(prof_foscrit.replace('\n', '')) #extracting critical factor of safety or slope failure probability
                sumfile.close() #closing file


    #Reading maximum number of layers for one ellipsoid

        if ( cflag or lflag ) and not mflag: #for single-core processing ellipsoid modes:
            sumfile=open(temppath+'/summary31.txt', 'r') #opening summary file for reading
            lbtmmaxmax=sumfile.readline() #reading maximum number of layers
            lbtmmaxmax=int(lbtmmaxmax.replace('\n', ''))
            lbtmalw=sumfile.readline() #reading maximum number of layers allowed
            lbtmalw=int(lbtmalw.replace('\n', ''))
            sumfile.close() #closing file


    #Writing documentation file

        if ( not vflag or ( cflag or iflag or lflag )) and not zflag: #if mode does not include confirmation only and is not test mode:
            documentation=open(pf+'_results/'+pf+'_files/'+pf+'_documentation.txt', 'w') #opening documentation file for writing

            if aflag: print>> documentation, 'a\t1'
            else: print>> documentation, 'a\t0'
            if cflag: print>> documentation, 'c\t1'
            else: print>> documentation, 'c\t0'
            if dflag: print>> documentation, 'd\t1'
            else: print>> documentation, 'd\t0'
            if iflag: print>> documentation, 'i\t1'
            else: print>> documentation, 'i\t0'
            if lflag: print>> documentation, 'l\t1'
            else: print>> documentation, 'l\t0'
            if mflag: print>> documentation, 'm\t1'
            else: print>> documentation, 'm\t0'
            if pflag: print>> documentation, 'p\t1'
            else: print>> documentation, 'p\t0'
            if sflag: print>> documentation, 's\t1'
            else: print>> documentation, 's\t0'
            if tflag: print>> documentation, 't\t1'
            else: print>> documentation, 't\t0'
            print>> documentation, 'cellsize\t%s' %cellsize

            if adm: #extracting area of detail map
                adm=map(str, adm.split(','))
                admN=adm[0]
                admS=adm[1]
                admW=adm[2]
                admE=adm[3]
                print>> documentation, 'admN\t%s' %admN
                print>> documentation, 'admS\t%s' %admS
                print>> documentation, 'admW\t%s' %admW
                print>> documentation, 'admE\t%s' %admE
            else:
                print>> documentation, 'admN\tNA'
                print>> documentation, 'admS\tNA'
                print>> documentation, 'admW\tNA'
                print>> documentation, 'admE\tNA'

            if elldens: print>> documentation, 'elldens\t%s' %elldens
            else: print>> documentation, 'elldens\tNA'
            if observed: print>> documentation, 'observed\t%s' %observed
            else: print>> documentation, 'observed\tNA'
            print>> documentation, 'elevation\t%s' %elevation
            print>> documentation, 'soilclass\t%s' %soilclass
            if slopeunits: print>> documentation, 'slopeunits\t%s' %slopeunits
            else: print>> documentation, 'slopeunits\tNA'
            if dxl: print>> documentation, 'dxl\t%s' %dxl
            else: print>> documentation, 'dxl\tNA'
            if dyl: print>> documentation, 'dyl\t%s' %dyl
            else: print>> documentation, 'dyl\tNA'
            if numlayers: print>> documentation, 'numlayers\t%s' %numlayers
            else: print>> documentation, 'numlayers\tNA'
            if depthmaps: print>> documentation, 'depthmaps\t%s' %depthmaps
            else: print>> documentation, 'depthmaps\tNA'
            if depthvals: print>> documentation, 'depthvals\t%s' %depthvals
            else: print>> documentation, 'depthvals\tNA'
            if prelayers: print>> documentation, 'prelayers\t%s' %prelayers
            else: print>> documentation, 'prelayers\tNA'
            if classlayers: print>> documentation, 'classlayers\t%s' %classlayers
            else: print>> documentation, 'classlayers\tNA'
            if maxlayers: print>> documentation, 'maxlayers\t%s' %maxlayers
            else: print>> documentation, 'maxlayers\tNA'
            print>> documentation, 'geotech\t%s' %geotech
            print>> documentation, 'seepage\t%s' %seepage
            if ellips: print>> documentation, 'ellips\t%s' %ellips
            else: print>> documentation, 'ellips\tNA'
            if sampling: print>> documentation, 'sampling\t%s' %sampling
            else: print>> documentation, 'sampling\tNA'
            print>> documentation, 'segsize\t%s' %segsize
            print>> documentation, 'nsegs\t%s' %nsegs
            if tilesx0: print>> documentation, 'tilesx\t%s' %tilesx0
            else: print>> documentation, 'tilesx\tNA'
            if tilesy0: print>> documentation, 'tilesy\t%s' %tilesy0
            else: print>> documentation, 'tilesy\tNA'
            if overlap0: print>> documentation, 'overlap\t%s' %overlap0
            else: print>> documentation, 'overlap\tNA'
            if ncores0: print>> documentation, 'ncores\t%s' %ncores0
            else: print>> documentation, 'ncores\tNA'

            print>> documentation, 'number of layers\t%i' %d_numlayers
            if ( cflag or lflag ) and mflag: print>> documentation, 'number of valid tiles\t%i' %jidval
            else: print>> documentation, 'number of valid tiles\t0'

            if ( cflag or lflag ) and elldens=='0':
                print>> documentation, 'number of layers\t%i' %prof_numlayers
                print>> documentation, 'id of critical layer\t%i' %prof_slipcrit
                print>> documentation, 'critical factor of safety\t%.4f' %prof_foscrit

            if ( cflag or lflag ) and not elldens=='0':
                print>> documentation, 'maximum y value of evolution plot\t%.5f' %status_max

            documentation.close() #closing documentation file


#confirmation and visualization

    if vflag:

        print
        print '3. CONFIRMING AND VISUALIZING MODEL RESULTS.'
        print

        if not cflag and not iflag and not lflag: #if mode only includes confirmation and visualization:


        #Reading documentation file for flags and options

            documentation=open(pf+'_results/'+pf+'_files/'+pf+'_documentation.txt', 'r') #opening documentation file

            aflag=documentation.readline()
            aflag=aflag.replace('a\t', '').replace('\n', '')
            cflag=documentation.readline()
            cflag=cflag.replace('c\t', '').replace('\n', '')
            documentation.readline()
            iflag=documentation.readline()
            iflag=iflag.replace('i\t', '').replace('\n', '')
            lflag=documentation.readline()
            lflag=lflag.replace('l\t', '').replace('\n', '')
            mflag=documentation.readline()
            mflag=mflag.replace('m\t', '').replace('\n', '')
            pflag=documentation.readline()
            pflag=pflag.replace('p\t', '').replace('\n', '')
            documentation.readline()
            documentation.readline()
            cellsize=documentation.readline()
            cellsize=cellsize.replace('cellsize\t', '').replace('\n', '')
            admN=documentation.readline()
            admN=admN.replace('admN\t', '').replace('\n', '')
            admS=documentation.readline()
            admS=admS.replace('admS\t', '').replace('\n', '')
            admW=documentation.readline()
            admW=admW.replace('admW\t', '').replace('\n', '')
            admE=documentation.readline()
            admE=admE.replace('admE\t', '').replace('\n', '')
            elldens=documentation.readline()
            elldens=elldens.replace('elldens\t', '').replace('\n', '')
            observed=documentation.readline()
            observed=observed.replace('observed\t', '').replace('\n', '')
            elevation=documentation.readline()
            elevation=elevation.replace('elevation\t', '').replace('\n', '')
            soilclass=documentation.readline()
            soilclass=soilclass.replace('soilclass\t', '').replace('\n', '')
            slopeunits=documentation.readline()
            slopeunits=slopeunits.replace('slopeunits\t', '').replace('\n', '')
            documentation.readline()
            documentation.readline()
            documentation.readline()
            documentation.readline()
            documentation.readline()
            documentation.readline()
            documentation.readline()
            documentation.readline()
            documentation.readline()
            documentation.readline()
            ellips=documentation.readline()
            ellips=ellips.replace('ellips\t', '').replace('\n', '')
            documentation.readline()
            documentation.readline()
            documentation.readline()
            documentation.readline()
            documentation.readline()
            overlap=documentation.readline()
            overlap=overlap.replace('overlap\t', '').replace('\n', '')
            documentation.readline()

            if aflag=='0': aflag=None
            if cflag=='0': cflag=None
            if iflag=='0': iflag=None
            if lflag=='0': lflag=None
            if mflag=='0': mflag=None
            if pflag=='0': pflag=None
            if admN=='NA': adm=None
            else: adm='1'
            if elldens=='NA': elldens=None
            if observed=='NA': observed=None
            if slopeunits=='NA': slopeunits=None
            if overlap=='NA': overlap=None

            d_numlayers=documentation.readline()
            d_numlayers=d_numlayers.replace('number of layers\t', '').replace('\n', '')
            d_numlayers=int(d_numlayers)
            jidval=documentation.readline()
            jidval=jidval.replace('number of valid tiles\t', '').replace('\n', '')
            if ( cflag or lflag ) and mflag: jidval=int(jidval)

            if ( cflag or lflag ) and elldens=='0':
                prof_numlayers=documentation.readline()
                prof_numlayers=int(prof_numlayers.replace('number of layers\t', '').replace('\n', ''))
                prof_slipcrit=documentation.readline()
                prof_slipcrit=int(prof_slipcrit.replace('id of critical layer\t', '').replace('\n', ''))
                prof_foscrit=documentation.readline()
                prof_foscrit=float(prof_foscrit.replace('critical factor of safety\t', '').replace('\n', ''))

            if ( cflag or lflag ) and not elldens=='0':
                status_max=documentation.readline()
                status_max=float(status_max.replace('maximum y value of evolution plot\t', '').replace('\n', ''))

            documentation.close() #closing documentation file

            grass.run_command('g.region', res=cellsize) #updating pixel size
            grass.run_command('g.copy', rast=(elevation,'xelev'), overwrite=True, quiet=True) #temporary elevation map
            grass.run_command('g.copy', rast=(soilclass,'xsoilclass'), overwrite=True, quiet=True) #temporary soil class map

            vonly=1 #setting control for confirmation mode only to 1 (positive)


    #Preparing spatial data sets for confirmation and visualization

        if observed and overlap and not ( elldens == '0' ):
            #if observation raster map and overlap were specified and more than one ellipsoid was tested:

            xvalmin=w0+float(overlap) #boundaries for confirmation
            yvalmin=s0+float(overlap)
            xvalmax=e0-float(overlap)
            yvalmax=n0-float(overlap)

            grass.mapcalc('"xsoilclass"=if("xsoilclass">-1,"xsoilclass", 1/0)', overwrite=True, quiet=True) #cleaned soil class map
            grass.mapcalc('"xsoilclassinv"=isnull("xsoilclass")', overwrite=True, quiet=True) #inverted soil class map
            grass.run_command('r.buffer', flags='z', input='xsoilclassinv', output='xconfirmation000',
                distances=overlap, quiet=True, overwrite=True)
            grass.mapcalc('"xconfirmation00"=isnull("xconfirmation000")', overwrite=True, quiet=True)
            grass.mapcalc('"xconfirmation0"=if("xconfirmation00">0,1,1/0)', overwrite=True, quiet=True)
            grass.mapcalc('"xconfirmation"=if(x()<%s||x()>%s||y()<%s||y()>%s,1/0,"xconfirmation0")'
                %(str(xvalmin),str(xvalmax),str(yvalmin),str(yvalmax)), overwrite=True, quiet=True) #confirmation area map

            grass.run_command('r.to.vect', input='xconfirmation', output='d_confirmation',
                type='area', flags='v'+'s', quiet=True, overwrite=True) #temporary confirmation area map as vector
            grass.run_command('v.out.ogr', flags='c', input='d_confirmation', output=temppath,
                type='boundary', quiet=True, overwrite=True) #temporary confirmation area map as shapefile

            grass.run_command('g.copy', rast=(observed,'xobserved0'), overwrite=True, quiet=True)
            grass.mapcalc('"xobserved"="xconfirmation"*"xobserved0"', overwrite=True, quiet=True) #temporary observed landslides map
            grass.run_command('r.out.gdal', input='xobserved', output=temppath+'/observed.tif',
                format='GTiff', flags='c', quiet=True, overwrite=True) #exporting observed landslides raster map to tiff

            grass.mapcalc('"xlobserved"=if("xobserved0">0,1,null())', overwrite=True, quiet=True)
            grass.run_command('r.to.vect', input='xlobserved', output='d_lobserved',
                type='area', flags='v'+'s', quiet=True, overwrite=True) #temporary observed landslides map as vector
            grass.run_command('v.out.ogr', flags='c', input='d_lobserved', output=temppath,
                type="boundary", quiet=True, overwrite=True) #temporary observed landslides map as shapefile

        grass.run_command('r.relief', input='xelev', output='xhillshade',
            quiet=True, overwrite=True) #temporary hillshade map
        grass.run_command('r.out.gdal', input='xhillshade', output=temppath+'/hillshade.tif',
            format='GTiff', flags='c', quiet=True, overwrite=True) #hillshade as geotiff

        if pflag: pflag2 = '1'
        else: pflag2 = '0'


    #Map plots

        if aflag and ( cflag or lflag ): ellparam=map(str, ellips.split(',')) #extracting ellipsoid parameters
        if aflag or elldens=='0': maxdepth=float(grass.raster_info("%s_depth" %pf)["max"]) #reading maximum critical slip surface depth

        d_maxval=[0.0] #initializing list of maximum values for map plots
        d_minval=[0.0] #initializing list of minimum values for map plots
        d_maxlab=0.0 #initializing label of maximum value
        nummaps=1 #initializing counter for number of maps

        if iflag or not elldens=='0': #for infinite slope mode or modes with multiple ellipsoids:
            #factor of safety
            d_maxval.append(2.5)
            d_minval.append(0.5)
            nummaps=nummaps+1

        if pflag: #slope failure probability and standard deviation of fos
            d_maxval.append(1.0)
            d_minval.append(0.0)
            d_maxval.append(2.5)
            d_minval.append(0.0)
            nummaps=nummaps+2

        if aflag and ( cflag or lflag ): #additional maps with ellipsoid modes:
            d_maxval.append(float(ellparam[0])) #critical slip surface length
            d_minval.append(float(ellparam[1]))
            d_maxval.append(float(ellparam[2])) #critical slip surface width
            d_minval.append(float(ellparam[3]))
            d_maxval.append(float(ellparam[6])) #critical offset of ellipsoid centre
            d_minval.append(float(ellparam[7]))
            nummaps=nummaps+3

            if not pflag: #for factor of safety:
                d_maxval.append(1.0) #susceptibility index
                d_minval.append(0.0)
                nummaps=nummaps+1

        if aflag or elldens=='0':
            d_maxlab=float(maxdepth) #maximum depth of critical slip surface for display
            d_maxval.append(float(maxdepth))
            d_minval.append(0.0)
            nummaps=nummaps+1

        if lflag and aflag: #additional map with layer mode:
            d_maxval.append(float(d_numlayers)) #critical layer
            d_minval.append(1.0)
            nummaps=nummaps+1

        for i in range(1,nummaps): #loop over all map plots:

            d_interval=(d_maxval[i]-d_minval[i])/6 #splitting range of values into six classes
            d_lev0=str(d_minval[i])
            d_lev1=str(d_minval[i]+1*d_interval)
            d_lev2=str(d_minval[i]+2*d_interval)
            d_lev3=str(d_minval[i]+3*d_interval)
            d_lev4=str(d_minval[i]+4*d_interval)
            d_lev5=str(d_minval[i]+5*d_interval)
            d_lev6=str(d_maxval[i])

            if not pflag: #defining name of each map
                if i==1 and elldens=='0': d_map='depth'
                elif i==1: d_map='fos'
                elif i==2: d_map='lcrit'
                elif i==3: d_map='wcrit'
                elif i==4: d_map='zbcrit'
                elif i==5: d_map='sindex'
                elif i==6: d_map='depth'
                else: d_map='layer'

            else:
                if i==1: d_map='fos'
                elif i==2: d_map='pfail'
                elif i==3: d_map='fos_stdev'
                elif i==4: d_map='lcrit'
                elif i==5: d_map='wcrit'
                elif i==6: d_map='zbcrit'
                elif i==7: d_map='depth'
                else: d_map='layer'

            if observed: ctrl_observed='1' #control for availability of observation raster map
            else: ctrl_observed='0'

            if pflag: pflagi=1
            else: pflagi=0

            subprocess.call('Rscript %s/r.slope.stability.map.r %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s'
                %(scriptpath, temppath, pf, str(n0), str(s0), str(w0), str(e0), cellsize, d_map, d_lev0, d_lev1, d_lev2, d_lev3, d_lev4,
                d_lev5, d_lev6, str(i), '0', d_maxlab, '1', ctrl_observed, pflagi), shell=True) #overview maps with hillshade

            if adm: subprocess.call('Rscript %s/r.slope.stability.map.r %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s'
                %(scriptpath, temppath, pf, admN, admS, admW, admE, cellsize, d_map, d_lev0, d_lev1, d_lev2, d_lev3, d_lev4,
                d_lev5, d_lev6, str(i), '1', d_maxlab, '1', ctrl_observed, pflagi), shell=True) #detail maps with hillshade

            subprocess.call('Rscript %s/r.slope.stability.map.r %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s'
                %(scriptpath, temppath, pf, str(n0), str(s0), str(w0), str(e0), cellsize, d_map, d_lev0, d_lev1, d_lev2, d_lev3, d_lev4,
                d_lev5, d_lev6, str(i), '0', d_maxlab, '0', ctrl_observed, pflagi), shell=True) #overview maps without hillshade

            if adm: subprocess.call('Rscript %s/r.slope.stability.map.r %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s'
                %(scriptpath, temppath, pf, admN, admS, admW, admE, cellsize, d_map, d_lev0, d_lev1, d_lev2, d_lev3, d_lev4,
                d_lev5, d_lev6, str(i), '1', d_maxlab, '0', ctrl_observed, pflagi), shell=True) #detail maps without hillshade


    #ROC plot

        if ( cflag or lflag or ( iflag and pflag ) ) and observed and overlap and not ( elldens=='0' ) and ( aflag or pflag ):
            #for ellipsoid modes with observed landslides and overlap specified,
            #with slope failure probability or additional output:

            if not pflag: #susceptibility index
                grass.mapcalc('"xsindex111"=isnull("%s_sindex")' %pf, overwrite=True, quiet=True)
                grass.mapcalc('"xsindex11"=if("xsindex111"==1,0,"%s_sindex")' %pf, overwrite=True, quiet=True)
            else: #slope failure probability
                grass.mapcalc('"xsindex111"=isnull("%s_pfail")' %pf, overwrite=True, quiet=True)
                grass.mapcalc('"xsindex11"=if("xsindex111"==1,0,"%s_pfail")' %pf, overwrite=True, quiet=True)

            grass.mapcalc('"xsindex1"=if("xsindex11"==-9999,1/0,"xsindex11")', overwrite=True, quiet=True)
            grass.mapcalc('"xsindex"="xconfirmation"*"xsindex1"', overwrite=True, quiet=True)
            grass.mapcalc('"xsindex"=if(isnull("xobserved")==1,1/0,"xsindex")', overwrite=True, quiet=True) #temporary pfail/sindex map
            grass.mapcalc('"xobserved1"=if(isnull("xsindex")==1,1/0,"xobserved")', overwrite=True, quiet=True) #temporary observation map

            grass.run_command('r.out.gdal', input='xsindex', output=temppath+'/sindex.tif',
                format='GTiff', flags='c', quiet=True, overwrite=True) #exporting temporary sindex/pfail map to tiff
            grass.run_command('r.out.gdal', input='xobserved1', output=temppath+'/observed.tif',
                format='GTiff', flags='c', quiet=True, overwrite=True) #exporting temporary observation map to tiff

            subprocess.call('Rscript %s/r.slope.stability.roc.r %s %s --slave --quiet'
                %(scriptpath, temppath, pf), shell=True) #ROC plot relating sindex/pfail to observation


    #Radar chart

        if observed and overlap and not ( elldens=='0' ) and not pflag:
            #if observation and overlap were specified and more than one ellipsoid was tested for the factor of safety:

            grass.mapcalc('"xfosbin"=if("%s_fos"==9999,null(),if("%s_fos"<1,1,0))' %(pf, pf), overwrite=True, quiet=True) #binary fos map
            grass.mapcalc('"xobsbin"=if("xobserved">0,1,0)', overwrite=True, quiet=True) #binary observation map
            grass.mapcalc('"xtp"=if("xfosbin"==1&&"xobsbin"==1,4,if("xfosbin"==0&&"xobsbin"==0,3, \
                if("xfosbin"==1&&"xobsbin"==0,2,if("xfosbin"==0&&"xobsbin"==1,1,null()))))',
                overwrite=True, quiet=True) #combination of fos and observation map
            grass.run_command('r.stats', flags='c'+'n', separator='x', input='xtp', output='%s/confirmation.txt'
                %temppath, quiet=True, overwrite=True) #scores tp=4, tn=3, fp=2, fn=1

            valfile=open(temppath+'/confirmation.txt', 'r') #opening raw file with confirmation scores
            valfile_lines=sum(1 for line in valfile) #reading number of lines
            valfile.close() #closing file

            if valfile_lines<4: #if number of lines is smaller than 4 (at least one of the scores is 0):
                valfile=open(temppath+'/confirmation.txt', 'r') #opening file again
                lr=valfile.readline() #reading first line
                l1=lr.find('1x') #checking for fn score (if not found, -1 is returned)
                if l1>-1: lr=valfile.readline() #if found, reading next line
                l2=lr.find('2x') #checking for fp score
                if l2>-1: lr=valfile.readline() #if found, reading next line
                l3=lr.find('3x') #checking for tn score
                if l3>-1: lr=valfile.readline() #if found, reading next line
                l4=lr.find('4x') #checking for tp score
                valfile.close() #closing file again
            else:
                l1=1  #if number of lines is 4, setting all controls to positive (any value larger than -1 would be fine)
                l2=1
                l3=1
                l4=1

            valfile=open(temppath+'/confirmation.txt', 'r') #once more opening temporary file with confirmation scores
            if l1==-1: fn=float(0) #if no fn score was identified before, setting to 0
            else:
                fn=valfile.readline() #else, reading line
                fn=float(fn.replace('1x', '').replace('\n', '')) #extracting fn score
            if l2==-1: fp=float(0) #if no fp score was identified before, setting to 0
            else:
                fp=valfile.readline() #else, reading line
                fp=float(fp.replace('2x', '').replace('\n', '')) #extracting fp score
            if l3==-1: tn=float(0) #if no tn score was identified before, setting to 0
            else:
                tn=valfile.readline() #else, reading line
                tn=float(tn.replace('3x', '').replace('\n', '')) #extracting tn score
            if l4==-1: tp=float(0) #if no tp score was identified before, setting to 0
            else:
                tp=valfile.readline() #else, reading line
                tp=float(tp.replace('4x', '').replace('\n', '')) #extracting tp score
            valfile.close() #closing file for the last time

            total=tp+tn+fp+fn #sum of scores
            tpr=100*tp/total #tp rate in percent
            tnr=100*tn/total #tn rate in percent
            fpr=100*fp/total #fp ratte in percent
            fnr=100*fn/total #fn rate in percent
            tlr=100*tp/(tp+fn) #rate of true positive predictions out of all positive observations
            tnlr=100*tn/(tn+fp) #rate of true negative predictions out of all negative observations
            mpr=tpr+fpr #total rate of positive predictions
            mnr=tnr+fnr #total rate of negative predictions
            tr=tnr+tpr #total rate of true predictions
            fr=fnr+fpr #total rate of false predictions
            opr=tpr+fnr #rate of positive observations
            onr=tnr+fpr #rate of negative observations
 
            print "True positive rate\trTP\t%.1f%%" %tpr #printing tp, tn, fp and fn rates
            print "True negative rate\trTN\t%.1f%%" %tnr
            print "False positive rate\trFP\t%.1f%%" %fpr
            print "False negative rate\trFN\t%.1f%%" %fnr
            print

            resultfile=open(pf+'_results/'+pf+'_files/'+pf+'_confirmation.txt', 'w') #opening file for confirmation scores for writing
            print>> resultfile, 'Criterion\tShortcut\tValue' #writing file header
            print>> resultfile, 'Rate of positive observations\trOP\t%.1f%%' %opr
            print>> resultfile, 'Rate of negative observations\trON\t%.1f%%' %onr
            print>> resultfile, 'True positive rate\trTP\t%.1f%%' %tpr #writing rates to file
            print>> resultfile, 'True negative rate\trTN\t%.1f%%' %tnr
            print>> resultfile, 'False positive rate\trFP\t%.1f%%' %fpr
            print>> resultfile, 'False negative rate\trFN\t%.1f%%' %fnr
            print>> resultfile, 'True positive rate out of all positive observations\trTP/rOP\t%.1f%%' %tlr
            print>> resultfile, 'True negative rate out of all negative observations\trTN/rON\t%.1f%%' %tnlr
            print>> resultfile, 'Rate of total positive predictions\trMP\t%.1f%%' %mpr
            print>> resultfile, 'Rate of total negative predictions\trMN\t%.1f%%' %mnr
            print>> resultfile, 'Rate of true predictions\trT\t%.1f%%' %tr
            print>> resultfile, 'Rate of false predictions\trF\t%.1f%%' %fr
            resultfile.close() #closing file

            radiotable=open(temppath+'/radtable.txt', 'w') #opening temporary input file for radioplot of confirmation scores for writing
            print>> radiotable, '%.1f\t%.1f\t%.1f\t%.1f' %(opr, onr, opr, onr) #writing theoretical maxima to file
            print>> radiotable, '0\t0\t0\t0' #writing theoretical minima to file
            print>> radiotable, '%.1f\t%.1f\t%.1f\t%.1f' %(tpr, tnr, fnr, fpr) #writing scores obtained by model to file
            print>> radiotable, '%.1f\t%.1f\t0\t0' %(opr, onr) #writing theoretical scores for perfect prediction to file as reference
            print>> radiotable, '%.1f\t%.1f\t%.1f\t%.1f' %(mpr*opr/100, mnr*onr/100,
                mnr*opr/100, mpr*onr/100) #writing theoretical random scores to file as reference
            radiotable.close() #closing file

            subprocess.call('Rscript %s/r.slope.stability.rad.r %s %s --slave --quiet' %(scriptpath, temppath, pf), shell=True)
                #radar chart with confirmation scores

            sumtable=open('sumtable_scores.txt', 'a')
            sumtable.write(geotech[3]+'\t'+geotech[4]+'\t'+str(tpr)+'\t'+str(tnr)+'\t'+str(fpr)+'\t'+str(fnr)+'\n')
            sumtable.close()


    #Evolution plot

        if ( cflag or lflag ) and not elldens == '0': #for ellipsoid modes with more than one ellipsoid:

            if mflag: numtiles=jidval #number of tiles (for multi-core mode)
            else: numtiles=1 #number of tiles (for single-core mode)
            subprocess.call('Rscript %s/r.slope.stability.evolution.r %s %s %s %s --slave --quiet'
                %(scriptpath, pf, numtiles, pflag2, status_max), shell=True)


    #Profile plot

        if ( cflag or lflag ) and not pflag and elldens == '0': #for ellipsoid-based factor of safety with only one ellipsoid:
            subprocess.call('Rscript %s/r.slope.stability.profile.r %s %i %i %.3f --slave --quiet'
                %(scriptpath, pf, prof_numlayers, prof_slipcrit, prof_foscrit), shell=True)


    #Regression and scatterplot based on slope units

        if slopeunits and observed and overlap and not elldens == '0':
            #if slope units, observation and overlap are provided and more than one ellipsoid was tested:

            grass.run_command('g.copy', rast=(slopeunits,'xslopeunits'), overwrite=True, quiet=True) #temporary slope units map

            if not pflag: grass.mapcalc('"xmodint"=if(isnull("xfosbin")==1||isnull("xobserved")==1,1/0,int("xfosbin"*1000))',
                overwrite=True, quiet=True) #binary fos integer raster map
            else:
                grass.mapcalc('"xpfail"=if("%s_pfail"==-9999,null(),"%s_pfail")' %(pf, pf), overwrite=True, quiet=True) #cleaned pfail map
                grass.mapcalc('"xmodint"=if(isnull("xpfail")==1||isnull("xobserved")==1,1/0,int("xpfail"*1000))',
                    overwrite=True, quiet=True) #integer raster map of slope failure probability  
            grass.mapcalc('"xobsint"=if(isnull("xmodint")==1||isnull("xobserved")==1,1/0,int("xobserved"*1000))',
                overwrite=True, quiet=True) #integer raster map of observed landslides
            grass.mapcalc('"xall"=if(isnull("xmodint")==1||isnull("xobserved")==1,1/0,int(1))', overwrite=True, quiet=True)
                #integer raster map for cell count
            grass.mapcalc('"xunint"=int("xslopeunits")', overwrite=True, quiet=True) #integer raster map of slope units

            grass.run_command('r.statistics', base='xunint', cover='xobsint', method='average',
                output='xobsstat', overwrite=True, quiet=True) #average of observed landslides map per slope unit
            grass.run_command('r.statistics', base='xunint', cover='xmodint', method='average',
                output='xmodstat', overwrite=True, quiet=True) #average of prediction map per slope unit
            grass.run_command('r.statistics', base='xunint', cover='xall', method='sum',
                output='xallstat', overwrite=True, quiet=True) #cell count per slope unit

            grass.run_command('r.stats', flags='l'+'n', separator='x', input='xobsstat',
                output='%s/obsstat.txt' %temppath, quiet=True, overwrite=True) #file with average of observed landslides map
            grass.run_command('r.stats', flags='l'+'n', separator='x', input='xmodstat',
                output='%s/modstat.txt' %temppath, quiet=True, overwrite=True) #file with average of prediction map
            grass.run_command('r.stats', flags='l'+'n', separator='x', input='xallstat',
                output='%s/allstat.txt' %temppath, quiet=True, overwrite=True) #file with cell count

            valfile=open(temppath+'/obsstat.txt', 'r') #opening file with average of observed landslides per slope unit
            valfile_lines = sum(1 for line in valfile) #counting number of lines
            valfile.close() #closing file

            regtable=open(temppath+'/regtable.txt', 'w') #opening input table file for regression for writing

            obsfile=open(temppath+'/obsstat.txt', 'r') #again, opening file with average of observed landslides
            for val in range(1,valfile_lines): #loop over all lines:
                obs=obsfile.readline() #reading line
                obs=re.sub('\d*x', '', obs) #extracting desired value (second column)
                obs=(float(obs.replace('\n', '')))/1000 #removing newline and converting to float
                print>> regtable, '%.3f\t' %obs, #writing to input file for regression (first line)
            obsfile.close() #closing file

            print>> regtable, '\n',

            modfile=open(temppath+'/modstat.txt', 'r') #opening file with average of prediction map per slope unit
            for val in range(1,valfile_lines): #loop over all lines:
                mod=modfile.readline() #reading line
                mod=re.sub('\d*x', '', mod) #extracting desired value (second column)
                mod=(float(mod.replace('\n', '')))/1000 #removing newline and converting to float
                print>> regtable, '%.3f\t' %mod, #writing to input file for regression (second line)
            modfile.close() #closing file

            print>> regtable, '\n',

            alcmax=0 #initializing maximum slope unit area
            allfile=open(temppath+'/allstat.txt', 'r') #opening file with cell count per slope unit
            for val in range(1,valfile_lines): #loop over all lines:
                alc=allfile.readline() #reading line
                alc=re.sub('\d*x', '', alc) #extracting desired value (second column)
                alc=(float(alc.replace('\n', '')))/1000 #removing newline and converting to float
                if alc>alcmax: alcmax=alc #updating maximum slope unit area, if necessary
                print>> regtable, '%.3f\t' %alc, #writing to input file for regression (third line)
            allfile.close() #closing file

            print>> regtable, '\n',

            whtfile=open(temppath+'/allstat.txt', 'r') #again opening file with cell count per slope unit
            for val in range(1,valfile_lines): #loop over all lines:
                wht=whtfile.readline() #reading line
                wht=re.sub('\d*x', '', wht) #extracting desired value (second column)
                wht=(float(wht.replace('\n', '')))/1000 #removing newline and converting to float
                wht1=10*pow(wht/alcmax,0.5) #relative slope unit area
                wht2=5*pow(wht*float(cellsize)*float(cellsize)/1000,0.5) #absolute slope unit area
                print>> regtable, '%.3f\t' %wht2, #writing to input file for regression (fourth line)
            whtfile.close() #closing file

            print>> regtable, ''
            regtable.close() #closing input table file for regression
            
            subprocess.call('Rscript %s/r.slope.stability.reg.r %s %s %s --slave --quiet'
                %(scriptpath, temppath, pf, pflag2), shell=True) #regression and scatterplot


#Writing computing time to file

    if not vonly: #if mode does not only include confirmation and visualization:
        if not mflag: comptime_batch=0
        timefile=open(pf+'_results/'+pf+'_files/'+pf+'_time.txt', 'w') #opening time file for writing
        print>> timefile, '%s\t%.5f\t%.5f' %(pf, comptime, comptime_batch) #writing computing time to file
        timefile.close() #closing time file


#Cleaning file system, printing message of completion and exiting

    os.system('rm -rf '+temppath) #removing temporary directory
    grass.run_command('g.remove',flags='f',type='rast',pattern='r_*', quiet=True) #removing raw result maps
    grass.run_command('g.remove',flags='f',type='rast',pattern='x*', quiet=True) #removing temporary raster maps
    grass.run_command('g.remove',flags='f',type='vect',pattern='d_*', quiet=True) #removing temporary vector maps

    grass.run_command('g.region', flags='d') #resetting default region

    print
    if not vonly: print 'Completed in %.1f seconds (net computing time).' %comptime
    else: print 'Completed.'

    if ( cflag or lflag ) and not vonly: #for ellipsoid modes with multi-core processing:

        if lflag: lbtmalw = lbtmalw-1; #reducing number of allowed layers by one for layer mode
        if lbtmalw >= lbtmmaxmax: print 'Maximum number of layers for one ellipsoid: %i' %lbtmmaxmax

        else: #if too many layers for at least one ellipsoid, print warning message:

            print
            print 'NUMBER OF LAYERS FOR AT LEAST ONE ELLIPSOID EXCEEDS POSSIBLE MAXIMUM:'
            print 'MODEL RESULTS ARE INVALID!'
            print
            print 'Maximum number of layers: %i' %lbtmmaxmax
            print 'Maximum number allowed: %i' %lbtmalw

        print

    print 'Please find the collected results in the directory %s_results.' %pf
    print

    sys.exit() #exit

if __name__ == '__main__':
    options, flags=grass.parser()
    main()

